(function() {
	function Freewheel_Mraid() {
		/* these variables are publicly available but not encouraged to use. It is mainly for single direction data sync. */
		this.expandProperties = {width:0,height:0,useCustomClose:false,isModal:true};
		this.maxSize = {width:0,height:0};
		this.screenSize = {width:0,height:0};
		this.defaultPosition = {x:0,y:0,width:0,height:0};
		this.currentPosition = {x:0,y:0,width:0,height:0};
		this.supportsFeatures = {sms:false,tel:false,calendar:false,storePicture:false,inlineVideo:false};
		this.resizeProperties = {};
		this.orientationProperties = {allowOrientationChange:true,forceOrientation:'none'};
		this.state = 'loading';
		this.placementType = null;
		this.listeners = {};
		this.viewable = true;
		this._Update = function(state, viewable, placementType, expandPropertiesJson, resizePropertiesJson, maxSizeJson, screenSizeJson, defaultPositionJson, currentPositionJson, isResized) {
			var originalState = this.state;
			var viewableChanged = viewable!=this.viewable;
			var newCurrentPosition = JSON.parse(currentPositionJson);
			var sizeChanged = (newCurrentPosition.width != this.currentPosition.width) || (newCurrentPosition.height != this.currentPosition.height);
			this.state = state;
			this.viewable = viewable;
			this.placementType = placementType;
			this.expandProperties = JSON.parse(expandPropertiesJson);
			this.resizeProperties = JSON.parse(resizePropertiesJson);
			this.maxSize = JSON.parse(maxSizeJson);
			this.screenSize = JSON.parse(screenSizeJson);
			this.defaultPosition = JSON.parse(defaultPositionJson);
			this.currentPosition = newCurrentPosition;
			if (originalState == 'loading' && state == 'default') {
				this.dispatchEvent('ready');
			}
			if (viewableChanged) this.dispatchEvent('viewableChange', viewable);
			if (originalState != 'loading' && state != 'hidden' && sizeChanged) {
				this.dispatchEvent('sizeChange', newCurrentPosition.width, newCurrentPosition.height);
			}
			if ((originalState != state) || (state == 'resized' && originalState == 'resized' && isResized)) {
				this.dispatchEvent('stateChange',state);
			}
		};

		this._setSupportingFeatures = function (supportsSms, supportsTel, supportsCalendar, supportsStorePicture, supportsInlineVideo) {
			this.supportsFeatures = {sms:supportsSms, tel:supportsTel, calendar:supportsCalendar, storePicture:supportsStorePicture, inlineVideo:supportsInlineVideo};
		};

		this._log = function(s) {
			try {
				FW_LOGGER.log(s);
			} catch ( e ) {
				if (window.console !== undefined)
					window.console.log(s + ", error:" +e);
			}
		};

		this._debug = function(s) {
			try {
				FW_LOGGER.debug(s);
			} catch ( e ) {
				if (window.console !== undefined)
					window.console.log(s + ", error:" +e);
			}
		};

		/* MRAID Methods*/
		this.addEventListener = function (event, listener) {
			if (this.listeners[event] === undefined) {
				this.listeners[event] = [];
			}
			this.listeners[event].push(listener);
		};

		this.removeEventListener = function (event, listener) {
			if (this.listeners[event] instanceof Array) {
				var lis = this.listeners[event], i, len;
				if (lis !== null) {
					len = lis.length;
					for (i = 0; i < len; i++) {
						if (lis[i] === listener) {
							lis.splice(i, 1);
							break;
						}
					}
				}
			}
		};

		this.dispatchEvent = function (event) {
			if(event === undefined) return;
			if (this.listeners[event] instanceof Array) {
				var lis = this.listeners[event], i, len;
				len = lis.length;
				for (i = 0; i < len; i++) {
					try {
						if (lis[i]) {
							lis[i].apply(this, Array.prototype.slice.call(arguments,1));
						}
					} catch (e) {
						this._debug('Exception in dispatchEvent:event=' + event + ', exception=' + e.toString());
					}
				}
			}
		};

		this.onError = function(message, action) {
			this._debug("onError: message=" + message + ", action=" + action);
		};

		this.open = function (url) {
			try {
				FW_MRAID_RENDERER.mraidOpen(url.toString());
			} catch ( e ) {
				this.dispatchEvent('error', '' + e, 'open');
			}
		};

		this.close = function () {
			try {
				FW_MRAID_RENDERER.mraidClose();
			} catch ( e ) {
				this.dispatchEvent('error', '' + e, 'close');
			}
		};

		this.useCustomClose = function (o) {
			this.expandProperties.useCustomClose = !!o;
			try {
				FW_MRAID_RENDERER.mraidUseCustomClose(this.expandProperties.useCustomClose);
			} catch ( e ) {
				this.dispatchEvent('error', '' + e, 'close');
			}
		};

		this.expand = function (url) {
			try {
				if (url === undefined) {
					FW_MRAID_RENDERER.mraidExpand();
				} else {
					FW_MRAID_RENDERER.mraidExpand(url.toString());
				}
			} catch ( e ) {
				this.dispatchEvent('error', '' + e, 'expand');
			}
		};

		this.setExpandProperties = function (o) {
			this.expandProperties.width = ((typeof o === 'undefined') || (typeof o.width === 'undefined'))?0:Math.round(o.width);
			this.expandProperties.height = ((typeof o === 'undefined') || (typeof o.height === 'undefined'))?0:Math.round(o.height);
			this.expandProperties.useCustomClose = ((typeof o === 'undefined') || (typeof o.useCustomClose === 'undefined'))?false:(!!o.useCustomClose);

			try {
				FW_MRAID_RENDERER.setExpandProperties(JSON.stringify(this.expandProperties));
			} catch ( e ) {
				this.dispatchEvent('error', '' + e, 'setExpandProperties');
			}
		};

		this.detachBridgeInterface = function() {
			this.open = this.close = this.expand = this.setExpandProperties = function () {};
			this._log = this._debug = function() {};
		};

		this.getExpandProperties = function () {
			return this.expandProperties;
		};

		this.isViewable = function () {
			return this.viewable;
		};
		this.getState = function () {
			return this.state;
		};

		this.getVersion = function () {
			return "2.0";
		};

		this.getPlacementType = function () {
			/* possible values: "inline", "interstitial". */
			/* inline: the ad placement is inline with content (i.e. a banner) in the display */
			/* interstitial: the ad placement is over laid on top of content */
			return this.placementType;
		};

		this.createCalendarEvent = function (o) {
			try {
				FW_MRAID_RENDERER.createCalendarEvent(JSON.stringify(o));
			} catch ( e ) {
				this.dispatchEvent('error', '' + e, 'createCalendarEvent');
			}
		};

		this.getOrientationProperties = function () {
			return this.orientationProperties;
		};

		this.setOrientationProperties = function (o) {
			this.dispatchEvent('error', 'Not supported', 'setOrientationProperties');
		};

		this.getCurrentPosition = function () {
			return this.currentPosition;
		};

		this.getDefaultPosition = function () {
			return this.defaultPosition;
		};

		this.getMaxSize = function () {
			return this.maxSize;
		};

		this.getResizeProperties = function () {
			return this.resizeProperties;
		};

		this.setResizeProperties = function (o) {
			if( (typeof o === 'undefined')
				|| (typeof o.width === 'undefined') || (typeof o.height === 'undefined')
				|| (typeof o.offsetX === 'undefined') || (typeof o.offsetY === 'undefined')) {
				this.dispatchEvent('error', 'Missing required parameters', 'setResizeProperties');
				return;
			}

			this.resizeProperties.width = Math.round(o.width);
			this.resizeProperties.height = Math.round(o.height);
			this.resizeProperties.offsetX = Math.round(o.offsetX);
			this.resizeProperties.offsetY = Math.round(o.offsetY);
			this.resizeProperties.customClosePosition = (typeof o.customClosePosition === 'undefined')?'top-right':o.customClosePosition;
			this.resizeProperties.allowOffscreen = ((typeof o.allowOffscreen !== 'undefined') && (o.allowOffscreen === false))?false:true;

			try {
				FW_MRAID_RENDERER.setResizeProperties(JSON.stringify(this.resizeProperties));
			} catch ( e ) {
				this.dispatchEvent('error', '' + e, 'setResizeProperties');
			}
		};

		this.resize = function () {
			try {
				FW_MRAID_RENDERER.mraidResize();
			} catch ( e ) {
				this.dispatchEvent('error', '' + e, 'resize');
			}
		};

		this.getScreenSize = function () {
			return this.screenSize;
		};

		this.playVideo = function (uri) {
			try {
				FW_MRAID_RENDERER.playVideo(uri.toString());
			} catch ( e ) {
				this.dispatchEvent('error', '' + e, 'playVideo');
			}
		};

		this.storePicture = function (uri) {
			this.dispatchEvent('error', 'Not supported', 'storePicture');
		};

		this.supports = function (feature) {
			return !!this.supportsFeatures[feature];
		};

		this.addEventListener('error',this.onError);
	}

	/* Initialize global mraid variable, to be used by ad. */
	window.mraid = new Freewheel_Mraid();
})();
